﻿using System.Collections.Generic;
using System.Drawing;

namespace Clash_Client
{
    public enum WindowPosition : byte { None, TopLeft, TopMiddle, TopRight, MiddleLeft, Centre, MiddleRight, BottomLeft, BottomMiddle, BottomRight }
    public delegate void DefaultDelegate();

    class Window
    {
        #region Fields
        public static List<Window> WindowList = new List<Window>();
        public bool Moveable, Dialog;
        public Size S;
        public WindowPosition StartLocation;
        public DefaultDelegate DrawBefore, DrawAfter;
        public string Text;
        public List<Button> ButtonList = new List<Button>();
        public List<TextBox> TextBoxList = new List<TextBox>();
        public List<ItemGrid> ItemGridList = new List<ItemGrid>();

        private Point DragPoint, location;
        private bool Initialized, visible;
        private int image;
        #endregion
        #region Properties

        public int Image
        {
            get { return image; }
            set
            {
                image = value;
                Initialized = false;
            }
        }
        public bool Visible
        {
            get { return visible; }
            set
            {
                visible = value;
                if (value)
                    WindowList.Insert(0, this);
                else
                    WindowList.Remove(this);
            }
        }
        public Point Location
        {
            get { return location; }
            set
            {
                if (value.X < 0) value.X = 0;
                else if (Point.Add(value, S).X >= Config.CScreenSize.Width) value.X = Config.CScreenSize.Width - 1 - S.Width;
                if (value.Y < 0) value.Y = 0;
                else if (Point.Add(value, S).Y >= Config.CScreenSize.Height) value.Y = Config.CScreenSize.Height - 1 - S.Height;
                if (location != value)
                {
                    location = value;
                    UpdateChildLocations();
                }
            }
        }

        #endregion
        #region Constructor
        public Window(int I, Point L, bool V)
        {
            Image = I;
            Location = L;
            Visible = V;
        }
        public Window(int I, Point L, bool V, bool M, bool D)
        {
            Image = I;
            Location = L;
            Visible = V;
            Moveable = M;
            Dialog = D;
        }
        public Window(int I, WindowPosition S, bool V)
        {
            Image = I;
            StartLocation = S;
            Visible = V;
        }
        public Window(int I, WindowPosition S, bool V, bool M, bool D)
        {
            Image = I;
            StartLocation = S;
            Visible = V;
            Moveable = M;
            Dialog = D;
        }
        #endregion
        public void Draw()
        {
            if (!Initialized) Initialize();
            if (Visible)
            {
                if (DrawBefore != null) DrawBefore.Invoke();
                CLib.Interface.Draw(Image, Location, false);
                if (DrawAfter != null) DrawAfter.Invoke();
                DrawChildControls();
            }
        }
        private void Initialize()
        {
            Initialized = true;
            S = CLib.Interface.GetImageSize(Image);
            switch (StartLocation)
            {
                case WindowPosition.TopLeft:
                    Location = Point.Empty;
                    UpdateChildLocations();
                    break;
                case WindowPosition.TopMiddle:
                    Location = new Point((Config.CScreenSize.Width - S.Width) / 2, 0);
                    break;
                case WindowPosition.TopRight:
                    Location = new Point((Config.CScreenSize.Width - S.Width), 0);
                    break;
                case WindowPosition.MiddleLeft:
                    Location = new Point(0, (Config.CScreenSize.Height - S.Height) / 2);
                    break;
                case WindowPosition.Centre:
                    Location = new Point((Config.CScreenSize.Width - S.Width) / 2, (Config.CScreenSize.Height - S.Height) / 2);
                    break;
                case WindowPosition.MiddleRight:
                    Location = new Point((Config.CScreenSize.Width - S.Width), (Config.CScreenSize.Height - S.Height) / 2);
                    break;
                case WindowPosition.BottomLeft:
                    Location = new Point(0, (Config.CScreenSize.Height - S.Height));
                    break;
                case WindowPosition.BottomMiddle:
                    Location = new Point((Config.CScreenSize.Width - S.Width) / 2, (Config.CScreenSize.Height - S.Height));
                    break;
                case WindowPosition.BottomRight:
                    Location = new Point((Config.CScreenSize.Width - S.Width), (Config.CScreenSize.Height - S.Height));
                    break;
                default:
                    UpdateChildLocations();
                    break;
            }
        }
        private void DrawChildControls()
        {
            foreach (Button B in ButtonList)
                B.Draw();
           foreach (TextBox T in TextBoxList)
                T.Draw();
            foreach (ItemGrid I in ItemGridList)
                I.Draw();
        }
        private void UpdateChildLocations()
        {
            foreach (Button B in ButtonList)
                B.UpdateLocation();
            foreach (TextBox T in TextBoxList)
                T.UpdateLocation();
            foreach (ItemGrid I in ItemGridList)
                I.UpdateLocation();

        }
        public void CleanUp()
        {
            foreach (Button B in ButtonList)
                B.CleanUp();

            foreach (TextBox T in TextBoxList)
                T.CleanUp();

            foreach (ItemGrid I in ItemGridList)
                I.CleanUp();

            Text = null;
            Visible = false;
        }

        public bool MouseDown(Point P)
        {
            if (new Rectangle(location, S).Contains(P))
            {
                P.Offset(-location.X, -location.Y);
                if (CLib.Interface.GetTransparent(image, P))
                {
                    WindowList.Remove(this);
                    WindowList.Insert(0, this);
                    foreach (Button B in ButtonList)
                        if (B.MouseDown(P)) return true;

                    foreach (TextBox T in TextBoxList)
                        if (T.MouseDown(P))
                        {
                            foreach (TextBox TT in TextBoxList)
                                if (TT != T) TT.Focus = false;
                            return true;
                        }
                    foreach (ItemGrid I in ItemGridList)
                        if (I.MouseDown(P)) return true;

                    if (Moveable && Engine.LeftDown)
                    {
                        P.Offset(location);
                        DragPoint.X = location.X - P.X;
                        DragPoint.Y = location.Y - P.Y;
                    }
                    return true;
                }
            }
            return Dialog;
        }
        public void MouseUp()
        {
            DragPoint = Point.Empty;
            foreach (Button B in ButtonList)
                B.MouseUp();
        }

        public bool MouseMove(Point P)
        {
            if (DragPoint != Point.Empty)
            {
                Location = new Point(P.X + DragPoint.X, P.Y + DragPoint.Y);
                return true;
            }
            P.Offset(-location.X, -location.Y);             
            foreach (Button B in ButtonList)
                if (B.MouseMove(P)) return true;

            foreach (TextBox T in TextBoxList)
                if (T.MouseMove(P)) return true;


            return false;
        }

        public bool Click(Point P)
        {
            if (new Rectangle(location, S).Contains(P))
            {
                P.Offset(-location.X, -location.Y);
                if (CLib.Interface.GetTransparent(image, P))
                {
                    foreach (Button B in ButtonList)
                        if (B.Click(P)) return true;

                    foreach (TextBox T in TextBoxList)
                        if (T.Click(P))
                        {
                            foreach (TextBox TT in TextBoxList)
                                if (TT != T) TT.Focus = false;
                            return true;
                        }

                    foreach (ItemGrid I in ItemGridList)
                        if (I.Click(P)) return true;

                    return true;
                }
            }
            return Dialog;
        }
        public bool DoubleClick(Point P)
        {
            if (new Rectangle(location, S).Contains(P))
            {
                P.Offset(-location.X, -location.Y);
                if (CLib.Interface.GetTransparent(image, P))
                {
                    foreach (Button B in ButtonList)
                        if (B.Click(P)) return true;

                    foreach (TextBox T in TextBoxList)
                        if (T.DoubleClick(P)) return true;

                    foreach (ItemGrid I in ItemGridList)
                        if (I.DoubleClick(P)) return true;

                    return true;
                }
            }
            return Dialog;
        }

        public bool RightClick(Point P)
        {
            if (new Rectangle(location, S).Contains(P))
            {
                P.Offset(-location.X, -location.Y);
                if (CLib.Interface.GetTransparent(image, P))
                {
                    foreach (ItemGrid I in ItemGridList)
                        if (I.RightClick(P)) return true;
                }
            }
            return Dialog;
        }
        public bool ContainsMouse(Point P)
        {
            return new Rectangle(Location, S).Contains(P) || Dialog;
        }


    }
}
